import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;

public class ImagePanel extends Panel implements MouseListener,MouseMotionListener{
  
  private Image image=null;
  private VisualByteMap visualByteMap;
  private static int maxImageWidth=300;
  private static int maxImageHeight=300;
  private int imageWidth,imageHeight;
  private int posX=0;    // VisualByteMap x offset in pixels
  private int posY=0;    // VisualByteMap y offset in pixels
  private int cursorX=0; // in sequence coordinates
  private int cursorY=0; //               "
  
  private UpdateTrigger updateTrigger;
  private boolean updateRequested=false;
  private boolean updating=false;
  private boolean messageDisplay=false;
  
  private int margin=5;
  private SeqPairPanel seqPairPanel;
  
  private String messageText="";
  private boolean  needRefresh=false;

  private Scrollbar xScrollbar;
  private Scrollbar yScrollbar;
  
  public ImagePanel(Component visibleComponent){
    image=visibleComponent.createImage(maxImageWidth,maxImageHeight);
    addMouseListener(this);
    addMouseMotionListener(this);
    updateTrigger=new UpdateTrigger(this);
    //updateTrigger.start();
    //displayMessage("loading");
  }
  public void setSeqPairPanel(SeqPairPanel spp){
    seqPairPanel=spp;
  }
  public int getImageWidth(){
    return imageWidth;
  }
  public int getImageHeight(){
    return imageHeight;
  }
  public int getImageX(){
    return posX;
  }
  public int getImageY(){
    return posY;
  }
  public Dimension getPreferredSize(){
    return new Dimension(maxImageWidth+2*margin,maxImageHeight+2*margin);
  }
  public Dimension getMinimumSize(){
    return getPreferredSize();
  }
  public void mouseClicked(MouseEvent e){}
  public void mouseEntered(MouseEvent e){}
  public void mouseExited(MouseEvent e){}
  public void mouseReleased(MouseEvent e){}
  public void mouseMoved(MouseEvent e){}
  public void mousePressed(MouseEvent e){
    int x=e.getX()-margin;
    int y=e.getY()-margin;
    if(x<0||x>=imageWidth||y<0||y>=imageHeight){
      int decX=0;
      int decY=0;
      if(x<0){decX=-1+x/10;}
      else{decX=1+(x-imageWidth)/10;}
      if(y<0){decY=-1+y/10;}
      else{decY=1+(y-imageHeight)/10;}
      // System.out.println(decX+" "+decY);
      moveImageRel(decX,decY);
      xScrollbar.setValue(getImageX());
      yScrollbar.setValue(getImageY());
      return;
    }
    x=(posX+x)*visualByteMap.getZoom();
    y=(posY+y)*visualByteMap.getZoom();
    setCursor(x,y);
    seqPairPanel.setCursor(x,y);
  }
  public void mouseDragged(MouseEvent e){
    mousePressed(e);
  }
  public void setScrollbar(Scrollbar x,Scrollbar y){
    xScrollbar=x;
    yScrollbar=y;
  }
  public void setCursor(int x,int y){
    // x and y are in sequence coordinates
   //x/=visualByteMap.getZoom();
    //y/=visualByteMap.getZoom();
    // now in VisualByteMap coordinates
    if(messageDisplay){return;}
    if(x<0){return;}
    if(x>(visualByteMap.getFullWidth()-1)){return;}
    if(y<0){return;}
    if(y>(visualByteMap.getFullHeight()-1)){return;}
    seqPairPanel.setCursor(x,y);
    // System.out.println("setCursor "+x+","+y);
    cursorX=x;
    cursorY=y;
    requestUpdate();
  }
  public int getCursorX(){
    return cursorX;//*visualByteMap.getZoom();
  }
  public int getCursorY(){
    return cursorY;//*visualByteMap.getZoom();
  }
  public void moveImageRel(int decX,int decY){
    // decX and decY are in pixels!
    moveImageTo(posX+decX,posY+decY);
  }
  public void moveImageTo(int newX,int newY){
    // posX and posY are in pixels!
    if(image==null){return;}
    if(newX<0){newX=0;}
    if(newY<0){newY=0;}
    if(newX>visualByteMap.getWidth()-imageWidth){newX=visualByteMap.getWidth()-imageWidth;}
    if(newY>visualByteMap.getHeight()-imageHeight){newY=visualByteMap.getHeight()-imageHeight;}
    posX=newX;
    posY=newY;
    requestUpdate();
  }
  public void setVisualByteMap(VisualByteMap vbm){
    visualByteMap=vbm;
    imageWidth=Math.min(maxImageWidth,vbm.getWidth());
    imageHeight=Math.min(maxImageHeight,vbm.getHeight());
  }
  public void paint(Graphics g){
    if(needRefresh){
      g.setColor(Color.lightGray);
      g.fillRect(margin,margin,maxImageWidth+1,maxImageHeight+1);
      needRefresh=false;
    }
    if(messageDisplay){
      g.setColor(Color.lightGray);
      g.fillRect(margin,margin,maxImageWidth,maxImageHeight);
      g.setColor(Color.black);
      g.drawRect(margin,margin,maxImageWidth,maxImageHeight);
      FontMetrics fm=g.getFontMetrics();
      int x=fm.stringWidth(messageText);
      g.drawString(messageText,(maxImageWidth-x)/2,maxImageHeight/2);
      //System.out.println("message DONE");
    }
    else{//display dot plot
      g.drawImage(image,margin,margin,this);
      g.setColor(Dotlet.BRIGHT_BLUE);
      int x=margin+cursorX/visualByteMap.getZoom()-posX;
      int y=margin+cursorY/visualByteMap.getZoom()-posY;
      if(x>margin-1 && x<margin+imageWidth){ 
	 g.drawLine(x,margin,x,margin+imageHeight-1);
      }
      if(y>margin-1 && y<margin+imageHeight){ 
	g.drawLine(margin,y,margin+imageWidth-1,y); 
      }
    }
    updating=false;
  }
  public void update(Graphics g){ // cancel background refresh for non blinking animation
    paint(g);
  }
  public void requestUpdate(){
    updateRequested=true;
  }
  void updateDisplay(){
    
    updateRequested=false; // set to false before updating allows
    // new update request to be catched!
    updating=true;
    if(messageDisplay){
      System.out.println(messageText);
    }
    else{
      visualByteMap.imageUpdate(image,posX,posY,imageWidth,imageHeight,this);
    }
    repaint();	     
  }
  boolean needUpdate(){
    return updateRequested && !updating;
  }
  public void displayDotPlot(){
    needRefresh=true;
    visualByteMap.prepareScale();
    image=createImage(imageWidth,imageHeight);
    messageDisplay=false;
    posX=0;  
    posY=0;   
    cursorX=0; 
    cursorY=0;
    if(updateTrigger.isAlive()){
      updateTrigger.stop();
    }
    updateTrigger=new UpdateTrigger(this);
    updateTrigger.start();
    requestUpdate();
  }
  public void displayMessage(String text){
    // System.out.println("message: "+text);
    if(updateTrigger.isAlive()){
      updateTrigger.stop();
    }
    messageText=text;
    needRefresh=true;
    messageDisplay=true;
    updateDisplay();
    // updating=false;
 
    //requestUpdate();
    // repaint();// and it does not repaint!!! I know
    //updateDisplay();
  }
}
class UpdateTrigger extends Thread{
  
  private ImagePanel imagePanel;
  private int delay=50;
  private int count=0;
  
  UpdateTrigger(ImagePanel p){
    imagePanel=p;
  }
  public void run(){
    try{
      while(true){
	sleep(delay);
	//System.out.println(count);
	count++;
	if(imagePanel.needUpdate()){
	  imagePanel.updateDisplay();
	}
      }
    }
    catch(InterruptedException e){
    }
  }
}

