import java.awt.*;
import java.awt.event.*;
import java.util.*;

public class InputPanel extends Panel implements ActionListener,ItemListener{
    
  private VisualByteMap visualByteMap;
  private DotterPanel dotterPanel;
  private ScalePanel scalePanel;
  private SeqPairPanel seqPairPanel;
  private InfoPanel infoPanel;
  
  private Hashtable sequences=new Hashtable();
  private Button inputButton=new Button("input"); 
  private Choice horizChoice=new Choice();
  private Choice vertChoice=new Choice();
  private Choice methodChoice=new Choice();
  private Button computeButton=new Button("compute");
  private Choice widthChoice=new Choice();
  private Choice zoomChoice=new Choice();
  private ComparisonMatrix comparisonMatrix;

  private static final String[] matrixName={"Identity",
					    "Blosum100","Blosum65","Blosum62","Blosum45","Blosum30",
					    "PAM30","PAM80","PAM120","PAM160","PAM200","PAM220","PAM250",
					    "Gonnet"};
  private static final String X_IDENTITY="Identity";
  
  private int seqCounter=1;
  
  public InputPanel(VisualByteMap vbm,DotterPanel dp,ScalePanel sp,SeqPairPanel spp,InfoPanel ip){
    visualByteMap=vbm;
    dotterPanel=dp;
    scalePanel=sp;
    seqPairPanel=spp;
    infoPanel=ip;

    //setLayout(new GridLayout(1,6));
    setLayout(new FlowLayout());
    add(inputButton);
    inputButton.addActionListener(this);
    //updateSequenceChoice(horizChoice);
    horizChoice.addItem("#horizontal#");
    add(horizChoice);
    horizChoice.setEnabled(false);
    //updateSequenceChoice(vertChoice);
    vertChoice.addItem("# vertical #");
    add(vertChoice);
    horizChoice.addItemListener(this);
    vertChoice.addItemListener(this);
    vertChoice.setEnabled(false);
    add(methodChoice);
    methodChoice.setEnabled(false);
    methodChoice.addItem("#matrix choice#");
    for(int i=0;i<30;i++){widthChoice.add(String.valueOf(1+2*i));}
    add(widthChoice);
    widthChoice.select("15");
    widthChoice.setEnabled(false);
    for(int i=1;i<11;i++){zoomChoice.add("1:"+String.valueOf(i));}
    add(zoomChoice);
    zoomChoice.setEnabled(false);
    add(computeButton);
    computeButton.addActionListener(this);
    computeButton.setEnabled(false);
  }
  public void actionPerformed(ActionEvent e){
    if(e.getSource()==inputButton){
      SeqInputDialog dialog=new SeqInputDialog(this);
    }
    if(e.getSource()==computeButton){
      compute();
    }
  }
  // called by the two sequence button!
  public void itemStateChanged(ItemEvent e){
    // System.out.println(sequences.isEmpty());
    if(sequences.isEmpty()){
      return;
    }
    methodChoice.removeAll();
    String seq1=(String)sequences.get(horizChoice.getSelectedItem());
    String seq2=(String)sequences.get(vertChoice.getSelectedItem());
    if(DNA.isDNA(seq1) && DNA.isDNA(seq2)){
      methodChoice.addItem(X_IDENTITY);
    }
    else{
      for(int i=0;i<matrixName.length;i++){methodChoice.add(matrixName[i]);}
      methodChoice.select("Blosum62");
    }
    methodChoice.addNotify();
  }	    
 private void updateSequenceChoice(Choice choice){
   if(sequences.isEmpty()){
     return;
   }
   choice.removeAll();
   for (Enumeration e = sequences.keys() ; e.hasMoreElements() ;) {
     choice.addItem((String)e.nextElement());
   }
   choice.addNotify();
 }
  private boolean inputIsValid(){
    if(horizChoice.getItemCount()==0){return false;}
    if(vertChoice.getItemCount()==0){return false;}
    return true;
  }
  public void addSequence(String name,String sequence){
    if(name.length()==0){
      name="seq_"+String.valueOf(seqCounter++);
    }
    if(sequence.length()==0){
      dotterPanel.displayMessage("Sequence is null");
      return;
    }
    dotterPanel.displayMessage("Sequence recorded with name "+name);
    sequences.put(name,sequence);
    updateSequenceChoice(horizChoice);
    updateSequenceChoice(vertChoice);
    itemStateChanged(null);
    horizChoice.setEnabled(true);
    vertChoice.setEnabled(true);
    methodChoice.setEnabled(true);
    computeButton.setEnabled(true);
    widthChoice.setEnabled(true);
    zoomChoice.setEnabled(true);
    repaint();
  }
    private void compute(){
	if(sequences.isEmpty()){
	    dotterPanel.displayMessage("FIRST input sequence");
	    return;
	}	    
	dotterPanel.displayMessage("computing"); // does not work
	String horizontalName=horizChoice.getSelectedItem();
	String horizontalSequence=(String)sequences.get(horizontalName);
	String verticalName=vertChoice.getSelectedItem();
	String verticalSequence=(String)sequences.get(verticalName);
	int windowSize=Integer.parseInt(widthChoice.getSelectedItem());
	int halfWindowSize=windowSize/2;
	String methodStr=methodChoice.getSelectedItem();
 	String zoomStr=zoomChoice.getSelectedItem();
	int zoomValue=Integer.parseInt(zoomStr.substring(2));
      	if(windowSize>horizontalSequence.length()){
	  dotterPanel.displayMessage(windowSize+" is too large");
	  return;
	}
	if(windowSize>verticalSequence.length()){
	  dotterPanel.displayMessage(windowSize+" is too large");
	  return;
	}
	dotterPanel.displayMessage("computing "+methodStr);
	try{
	  comparisonMatrix=(ComparisonMatrix)Class.forName(methodStr).newInstance();
	}
	catch(Exception e){
	  dotterPanel.displayMessage(e.toString());
	  return;
	}
	visualByteMap.setZoom(zoomValue);
	ScoreTable scoreTable=null;
	try{
	  if(DNA.isDNA(horizontalSequence)){
	    if(DNA.isDNA(verticalSequence)){ //DNA vs DNA
	      scoreTable=new DNADNAScoreTable(horizontalSequence,
					      verticalSequence,
					      halfWindowSize,
					      comparisonMatrix,
					      visualByteMap);
	    }
	    else{ //DNA vs PROT
	      if(windowSize>horizontalSequence.length()/3-2){
		dotterPanel.displayMessage(windowSize+" is too large");
		return;
	      }
	      scoreTable=new DNAProteinScoreTable(horizontalSequence,
						  verticalSequence,
						  halfWindowSize,
						  comparisonMatrix,
						  visualByteMap);		    
	    }
	  }
	  else{
	    if(DNA.isDNA(verticalSequence)){ //PROTvs DNA
	      dotterPanel.displayMessage("try DNA vs PROT");
	    }
	    else{ //PROT vs PROT
	      scoreTable=new SameTypeScoreTable(horizontalSequence,
						verticalSequence,
						halfWindowSize,
						comparisonMatrix,
						visualByteMap);      
	    }
	  }
	}
	catch(UnknownCodonException e){
	  System.out.println("unknown codon");
	  dotterPanel.displayMessage("unknown codon");
	  scoreTable=null;
	}
	catch(IdioticUserChoiceException e){
	  System.out.println("unknown error");
	  dotterPanel.displayMessage("unknown error");
	  scoreTable=null;
	}
	catch(java.lang.OutOfMemoryError e){
	  dotterPanel.displayMessage("OUT OF MEMORY");
	  scoreTable=null;
	}
	if(scoreTable!=null){
	  //dotterPanel.displayMessage("computing");
	  //dotterPanel.repaint();
	  //Thread.sleep(1000);
	  // dotterPanel.repaint(1);
	  // fill the dotplot
	  seqPairPanel.setEnabled(false);
	  scalePanel.setEnabled(false);
	  Thread thread=new Thread(scoreTable);
	  //thread.setPriority(Thread.MIN_PRIORITY);
	  //System.out.println("START");
	  thread.start();
	  try{
	    thread.join();
	  }
	  catch(Exception e){}
	  //while(thread.isAlive()){
	  //System.out.println("Thread is alive");
	  //  dotterPanel.displayMessage("computing "+methodStr);
	  
	  //Thread.sleep(100);
	  //}
	  // System.out.println("STOP");
	  try{
	    seqPairPanel.setWidget(horizontalName,
				 horizontalSequence,
				 verticalName,
				 verticalSequence,
				 comparisonMatrix,
				 halfWindowSize);
	  }
	  catch(Exception e){}
	  dotterPanel.setVisualByteMap(visualByteMap);
	  dotterPanel.displayDotPlot();
	  dotterPanel.setCursor(0,0);
	  scalePanel.setEnabled(true);
	  scalePanel.updateScale();
	  seqPairPanel.setEnabled(true);
	  infoPanel.setInfo(horizontalName,verticalName,methodStr,String.valueOf(windowSize),zoomStr);
	}
	
    }
}

  
